/*
 * Project name:
     Serial 7-seg Display 2 AVR Demo Counter V1
 * Copyright:
     core code provided by  Mikroelektronika, 2010.
     Since I could not get it to work as written  I rewrote it, kept it simple,
     and commented each line for others to use.   - Mike Manteo - 3/29/2015
 * Description:
     This code demonstrates working with MAX7219 on Serial 7-seg Display2 board.
     Eight 7 segment display is connected to the MAX7219 ( 2x4 form factor ).
     The single row eight 7 Seg display (1x8) will work fine here.
 * Test configuration:
     mcu:              Atmega 32
     dev.board:        EasyAVR v7
     Oscillator:       Crystal 8.0 MHz
     ext. modules:     Serial 7-Seg Display 2
     sw:  mikroC Pro for AVR

     Notes:
        - Connect extra board to PortB
        - extra board Dip SW 3,6 and 7 are ON
        - Use +5 VDC supply
        - Pins assigned
            PB7   SCK     output SPI module
            PB5   MOSI    output SPI module
            PB2   MISO    input  SPI module
            PB0   CS0     output  bit bang - active LOW

     Undetected errors are ignored.

*/


// Serial 7-seg Display connections
sbit  Chip_Select  at PORTB.B0;           // ext. module Dip SW 7 ON
sbit  Chip_Select_Direction  at DDRB.B0;  // define a pin
// End Serial 7-seg Display connections

unsigned long int value;
unsigned long int data1;

void max7219_init1() {

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x09);          // index MODE
  SPI1_write(0xFF);          // value FF = BCD mode for digit decoding
  Chip_Select = 1;           // deselect max7219

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x0A);          // index Intensity
  SPI1_write(0x01);          // value 01 = Segment luminosity intensity - 00 is dim
  Chip_Select = 1;           // deselect max7219

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x0B);          // index Scan Limit
  SPI1_write(0x07);          // value 07 Display refresh - 0x00 one digit  0x07 all 8 digits
  Chip_Select = 1;           // deselect max7219

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x0C);          // index Display on - off
  SPI1_write(0x01);          // value 01 turns on the display - 00 turn off
  Chip_Select = 1;           // deselect max7219

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x0F);          // index Lamp Test
  SPI1_write(0x00);          // value 00 = No test   FF will do lamp test
  Chip_Select = 1;           // deselect max7219

  Chip_Select = 0;           // deselect max7219
  SPI1_write(0x00);          // index no op
  SPI1_write(0x00);          // value no op
  Chip_Select = 1;           // deselect max7219

}
void txt() {
                                  // --
                                  //HELP

  Chip_Select = 0;                // select max7219
  SPI1_write(0x08);               // index Position
  SPI1_write(0x0F);               // value ASCII blank
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x07);               // index Position
  SPI1_write(0x0A);               // value ASCII -
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x06);               // index Position
  SPI1_write(0x0A);               // value ASCII -
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x05);               // index Position
  SPI1_write(0x0F);               // value ASCII blank
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x04);               // index Position
  SPI1_write(0x0C);               // value ASCII H
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x03);               // index Position
  SPI1_write(0x0B);               // value ASCII E
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x02);               // index Position
  SPI1_write(0x0D);               // value ASCII L
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x01);               // index Position
  SPI1_write(0x80+0x0E);          // value ASCII . and P
  Chip_Select = 1;                // deselect max7219

  Chip_Select = 0;                // select max7219
  SPI1_write(0x00);               // index no op
  SPI1_write(0x00);               // value no op
  Chip_Select = 1;                // deselect max7219
  }




void display_number( unsigned long int data1) {    // (char) limit 255 (8bit)
                                                   // (int) limit 32767 (signed) (< 16 bit)
                                                   // (unsigned long int) (< 32 bit)

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x08);                 // index ten millions digit
  SPI1_write((data1/10000000)%10);  // value %10 limits display numbers 0-9
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x07);                 // index millions digit
  SPI1_write((data1/1000000)%10);   // value
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x06);                 // index hundred thousands digit
  SPI1_write((data1/100000)%10);    // value
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x05);                 // index thousands digit
  SPI1_write((data1/10000)%10);     // value
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x04);                 // index thousands digit
  SPI1_write((data1/1000)%10);      // value
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x03);                 // index hundreds digit
  SPI1_write((data1/100)%10);       // value
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x02);                 // index tens digit
  SPI1_write(0x80+(data1/10)%10);   // value 0x80 + foo will place a decimal  point
  Chip_Select = 1;                  // deselect max7219

  Chip_Select = 0;                  // select max7219
  SPI1_write(0x01);                 // index ones digit
  SPI1_write(data1%10);             // value
  Chip_Select = 1;                  // deselect max7219

}

void main() {


Chip_Select_Direction = 1;     // DDRB.B0 - Set CS# pin as Output  1 is  an output
SPI1_init();                   // call initialize SPI module function
Delay_ms(100);
max7219_init1();               // call initialize  max7219 function
Delay_ms(100);
txt();                         // call print refinery  function -- HELP
Delay_ms(5000);                // 5 second display



for (value=0; value<100000000; value++) {  // increment counter
   display_number(value);      // call digit refinery - display
    Delay_ms(100);             // .1 second of a second display
   }
  Chip_Select = 0;             // select max7219
  SPI1_write(0x0C);            // index Display on - off
  SPI1_write(0x00);            // value 01 turns on the display - 00 turns off
  Chip_Select = 1;             // deselect max7219

}